
#-----------------------------------------------------------------------------
#  model2  (ver. 0.1) -- Estimating Agglomeration Model 2
#  Copyright (C) 2007 Giulio Bottazzi and Angelo Secchi

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  (version 2) as published by the Free Software Foundation;
  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#------------------------------------------------------------------------------


import string
from math import *
from scipy import special
from scipy.optimize import fmin_bfgs
from scipy.optimize.minpack import leastsq
from scipy.stats import chi2
from scipy.stats import histogram2
from numpy import *
import shelve
import Scientific.IO.ArrayIO


#define polya probability function
#---------------------------------
def polya(n,b,beta):
    A=L
    rr=(1+(beta/L)*(r-1))/b
    prob=exp(special.gammaln(rr+n)-special.gammaln(rr)+special.gammaln(L/b-rr+N-n)-special.gammaln(L/b-rr)+special.gammaln(N+1.)-special.gammaln(n+1.)-special.gammaln(N-n+1.)+special.gammaln(A/b)-special.gammaln(A/b+N))
    prob=sum(prob)
    return prob/L


#theoretical frequencies from the Model
#--------------------------------------
def model(b,beta):
    freq      = [ polya(x,b,beta) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    classfreq.append(1.-sum(classfreq))
    return array(classfreq)


#Minimize Chis^2 using Levenberg-Marquardt alghoritm
#----------------------------------------------------

def thempdiff(par):
    b=exp(par[0])
    beta=exp(par[1])
    freq      = [ polya(x,b,beta) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    if sum(classfreq) != 1.0:
        classfreq=classfreq/sum(classfreq)
    classfreqsq = sqrt(array(classfreq))
    return classfreqsq - empfreq[:-1]/classfreqsq


def min_LM(thempdiff):
    results=leastsq(thempdiff,x0=init) #Levenberg-Marquardt algorithm
    xopt=exp(results[0])
    freq      = [ polya(x,xopt[0],xopt[1]) for x in range(nmax) ]
    classfreq = [ sum(freq[2**k-1:2**(k+1)-1]) for k in range(maxindex) ]
    if sum(classfreq) != 1.0:
        classfreq=classfreq/sum(classfreq)
    fss=sum(((classfreq - empfreq[:-1])**2)/classfreq)
    sav=sum(abs((classfreq - empfreq[:-1])))/maxindex
    pvalue=chi2.pdf(fss,maxindex-2)
    return xopt,pvalue,fss,sav,(classfreq*L)


#Main Loop
#----------


storage=shelve.open("shelve_init")
INIT=storage["INIT"]
storage.close()



storage=shelve.open("shelve_regression")
matricione=storage["matricione"]
storage.close()
nsec=len(matricione[1])

risultati_LM=[]
mod2=[]
emp={}
t2={}

for sec in range(nsec):
    storage=shelve.open("shelve_regression")
    matricione=storage["matricione"]
    storage.close()
    empdata=matricione[:,sec]
    L=len(matricione)
    N=sum(empdata)
    m=sum(matricione,1)-empdata
    M=sum(m)
    nmax= int(max(empdata))
    r=L*m/M
    maxindex=int(ceil(log(nmax+2)/log(2)-1))
    empfreq=histogram2(empdata,[ 2**k-1  for k in range(maxindex) ])/float(L)
    empfreq=hstack((empfreq,array(0.0)))
    #init=log([0.8,1.0])
    init=log(INIT)
    temp=min_LM(thempdiff)
    res=transpose(hstack((temp[0][0],temp[0][1],temp[3],temp[2])))
    risultati_LM.append(res)
    print '\n#######  SECTOR ',sec+1,' #######'
    print '\n Model 2 \n'
    print 'b =',res[0] , 'beta=', res[1] 
    print '\n AAD=', res[2], 'Chi^2=', res[3]
    emp[sec]=empfreq[:-1]*L
    t2[sec]=temp[4]



output=file("results/model2_coef.dat","w")
print >> output, "Coef. b \t Coeff. beta \t AAD \t Chi^2"
for i in risultati_LM:
     print >> output, i

output.close()




#Creating Shelve
#----------------------------

storage=shelve.open("shelve_model2")

storage['mod2']=mod2
storage['emp']=emp
storage['t2']=t2

storage.close()


storage=shelve.open("shelve_model2")

mod2=storage['mod2']
emp=storage['emp']
t2=storage['t2']

storage.close()


Emp=[]
T2=[]

for sec in  range(nsec):
    empirica=emp[sec]
    teorica2=t2[sec]
    Emp.append(empirica)
    T2.append(teorica2)


#1) Files per i Plots

output=file("results/model2_emp.txt","w")

for sec in Emp:
    print >> output, str(sec)[1:-1]

output.close()

output=file("results/model2_theo.txt","w")

for sec in T2:
    print >> output, str(sec).replace('\n','')[1:-1]

output.close()
