
#-----------------------------------------------------------------------------
#  polyloc (ver. 0.1) -- Estimating Agglomeration Models
#  Copyright (C) 2007 Giulio Bottazzi and Angelo Secchi

#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  (version 2) as published by the Free Software Foundation;
  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#------------------------------------------------------------------------------

import scipy
import sys
import getopt
import string
import os
import shelve

#set the default parameter
#-------------------------

MODEL='model1'
LOCATIONTYPEDATA='None'
LOCATIONS='inputdict.keys()'
locationslabel='None'
verbose = False
INIT='1.0,1.0'


optlist, args=getopt.getopt(sys.argv[1:], 'hve:m:t:i:')

for o, a in optlist:
    if o == "-v":
        verbose = True
    if o == "-e" and LOCATIONTYPEDATA!='None':
        locationslabel=a
        if a == 'cities':
            LOCATIONS='keys_data_nocity'
        elif a == 'metropolitan-areas':
            LOCATIONS='keys_data_nourban'
        elif a == 'districts':
            LOCATIONS='keys_data_nodistrict'
        else:
            print '\n Houston, Houston we have a problem!\n (ERROR: wrong effect name!)'
    if o in ("-m"):
        MODEL=a
    if o in ("-t"):
        LOCATIONTYPEDATA=a
    if o in ("-i"):
        INIT=a
    if o == '-h':
        print "bodofase.py estimates the Bottazzi-Dosi-Fagiolo-Secchi model of firms location "
        print "(see README for details). \n                                                "
        print "Usage:",sys.argv[0]," [options]\n"
        print "Options:"
        print " -t    Specify the data file with location type(cities, metropolitan        " 
        print "       areas, districts)                                        [none]      "
        print " -i    Specify initial conditions for b and beta             [1.0,1.0]      "
        print " -e    Remove the effect of:\n                                              "
        print "       cities     metropolitan-areas     districts              [none]    \n"
        print " -m    Choose the model:\n                                                  "
        print "       model0     model1     model2                           [model0]    \n"
        print " -v    produce verbose output                                               "
        print " -h    this help message                                                    "
        sys.exit()



if MODEL not in ("model0","model1","model2"):
    print >> sys.stderr ,"\n ERROR Model is wrongly specified \n "
    sys.exit()


#Read data and prepare them
#==========================


data={}
for line in sys.stdin:
    string.strip(line)
    if line[0] == '#' :
        continue
    splitted=string.split(line,',')
    if len(splitted) < 2 :
        continue
    key = splitted[0]
    if key[0]=='"' :
        key = key[1:]
    if key[-1]=='"' :
        key= key[0:-1]
    del splitted[0]
    data[key]=[]
    #print key,
    for piece in splitted:
        #print piece
        #print key
        if piece == '' or piece == '.' or piece == '.\n':
            data[key].append(0.0)
        else :
            data[key].append(float(piece))
    #print len(data[key])



if LOCATIONTYPEDATA != 'None':
    filename = eval('LOCATIONTYPEDATA')
    #print filename
    input = file(filename,"r")
    data_type={}
    for line in input.readlines():
        if string.find(line,"#") != -1 :
            continue
        splitted=string.split(line,',')
        #print splitted[0]
        key = splitted[0]
        if key[0]=='"' :
            key = key[1:]
        if key[-1]=='"' :
            key= key[0:-1]
        del splitted[0]
        data_type[key]=[]
        data_type[key].append(map(int,splitted[0:]))
        input.close()


storage=shelve.open("shelve")
storage['data']=data
if LOCATIONTYPEDATA != 'None':
    storage['data_type']=data_type

storage.close()



if LOCATIONTYPEDATA != 'None':
    keys=data.keys()
    keys.sort
    outfile = open(".partial.dat","w")
    for key in keys :
        for x in data[key]:
            outfile.write(str(x)+' ')
        outfile.write('\n')
    outfile.close()
    keys_data_nourban=[]
    for key in data.keys() :
        #print key
        if data_type[key][0][1]==1:
            continue
        if data_type[key][0][1]==0:
            keys_data_nourban.append(key)
        else:
            print 'Houston, Houston we have a problem!'
    keys_data_nocity=[]
    for key in data.keys():
        #print key
        #print data_type[key]
        if data_type[key][0][2]==1:
            continue
        if data_type[key][0][2]==0:
            keys_data_nocity.append(key)
        else:
            print 'Houston, Houston we have a problem!'
    keys_data_nodistrict=[]
    for key in data.keys() :
        if data_type[key][0][0]==1:
            continue
        if data_type[key][0][0]==0:
            keys_data_nodistrict.append(key)
        else:
            print 'Houston, Houston we have a problem!'
else:
    keys=data.keys()
    keys.sort
    outfile = open(".partial.dat","w")
    for key in keys :
        for x in data[key]:
            outfile.write(str(x)+' ')
        outfile.write('\n')
    outfile.close()


#Save the initial conditions
#===========================

pippo=string.split(INIT,',')
INIT=[float(pippo[0]),float(pippo[1])]
storage=shelve.open("shelve_init")
storage['INIT']=INIT
storage.close()


# MAIN - compute the useful quantities
#-------------------------------------
#DO not modify under this line

#n[l][i] number of Buisiness Unit of sector i in location l
#nloc[l] total number of BU in location l
#nsec[i] total number of BU of sector i


inputdict=data
locations=eval(LOCATIONS)
locations.sort()

if verbose :
    print >> sys.stderr ,"\n ==============  Model Specification  ========================"
    print >> sys.stderr ,"\n Model to fit is                           :", MODEL
    print >> sys.stderr, "\n Number of Locations                       :", len(locations)
    print >> sys.stderr ,"\n Removing the effect of                    :", locationslabel
    print >> sys.stderr ,"\n Initial conditions for the optimization   :", '%.2f %.2f' % (INIT[0], INIT[1])
    print >> sys.stderr, "\n ============================================================="

n=[]
nloc=[]
nsec=[0.0]*len(inputdict[locations[0]])
for loc in locations:
    n.append(inputdict[loc])
    tmpsum=0.0
    for val in inputdict[loc]:
        tmpsum=tmpsum+val
    nloc.append(tmpsum)
    nsec=map(lambda a,b: a+b,nsec,inputdict[loc])

ntot=reduce(lambda a,b : a+b,nloc)


pippa2=[]
for loc in range(0,len(locations)):
    pippa=n[loc][:]/scipy.array(nsec[:])
    pippa2.append(pippa)

matricione=scipy.vstack(pippa2)

outfile = open(".frequency.dat","w")
for row in range(0,len(matricione)):
    for col in range(0,len(scipy.transpose(matricione))):
        outfile.write(str(matricione[row,col]/scipy.sum(matricione[row]))+'\t')
    outfile.write('\n')

outfile.close()

pippa2=[]
for loc in range(0,len(locations)):
    pippa=n[loc][:]
    pippa2.append(pippa)

matricione=scipy.vstack(pippa2)

outfile = open(".number.dat","w")
for row in range(0,len(matricione)):
    for col in range(0,len(scipy.transpose(matricione))):
        outfile.write(str(matricione[row,col])+'\t')
    outfile.write('\n')

outfile.close()

storage=shelve.open("shelve_regression")
storage['matricione']=matricione
storage.close()


#Generating if necessary a directory for results

if os.path.isdir("results") == False:
    os.mkdir("results")


#Fitting model

if MODEL == 'model0':
    import model0

if MODEL == 'model1':
    import model1

if MODEL == 'model2':
    import model2
    
